/*

	SDL video backend for Windows and *nix
  
*/

#include "m_core.h"
#include "m_gfx.h"
#include "joystick.h"
#include "SDL/SDL.h"

// forward def of palette in RGBA format
extern unsigned char pal256[1024];

unsigned char Keys[128] = {0};
unsigned char ScreenBuffer[320*200] = {0};

// position of the joystick analog
int joyHasMovedX = 0;
int joyHasMovedY = 0;
int joyLastPosX = 0;
int joyLastPosY = 0;

// if we want to reverse the joystick axis
int joyReverseAxisX = 0;	// left is right if set to 1
int joyReverseAxisY = 0;	// up is down if set to 1

#if defined(__DINGUX__) || defined(__GP2X__) || defined(__CAANOO__)
	int scale2x = 1;
	int fullscr = 0;
	int bpp = 16;
	//int bpp = 8;
#else
	int scale2x = 2; // 1 - no scale 320x200; 2 - upscale to 640x400
	int fullscr = 0; // or SDL_FULLSCREEN
	int bpp = 8;
#endif

int _toggle = 1;

int LM_GFX_Init();
void LM_GFX_Deinit();

SDL_Surface *screen = NULL;
SDL_Surface *hardwareScreen = NULL;

//===============================================================
void LM_ResetKeys()
{
	memset(&Keys[0], 0, 128);
}

int LM_AnyKey()
{
    for(int i = 0; i < 127; i++)
	{
        if(Keys[i] == 1) return 1;
    }
    return 0;
}

int LM_Timer()
{
	return SDL_GetTicks();
}

void LM_Sleep(int sleep_time)
{
	SDL_Delay(sleep_time); 
}

int LM_Init(unsigned char **pScreenBuffer)
{
	if(LM_GFX_Init() == 0) return 0;
	*pScreenBuffer = &ScreenBuffer[0];
	return 1;
}

void LM_Deinit()
{
	LM_GFX_Deinit();
}

char LM_PollEvents()
{
	SDL_Event event;
   
	while (SDL_PollEvent(&event) != 0)
	{ 
		#ifndef __WIN32__
		int key_scan = -1;
		unsigned char key_value = 0;
		#endif

		if(event.type == SDL_QUIT) {Keys[SC_ESCAPE] = 1; return 1; }
		
		// only valid for i386 arch and Windows
		#ifdef __WIN32__
		if(event.type == SDL_KEYDOWN )
		{
			Keys[event.key.keysym.scancode] = 1;
		} else if(event.type == SDL_KEYUP)
		{
			Keys[event.key.keysym.scancode] = 0;
		}
		#endif

		// unix and dingoo sdl don't have scancodes, so remap usual keys
		#if defined(__UNIX__) || defined(__DINGOO__) || defined(__DINGUX__) || defined(__GP2X__) || defined(__CAANOO__)
		if(event.type == SDL_KEYDOWN) key_value = 1;
		if(event.type == SDL_KEYUP) key_value = 0;
		if(event.type == SDL_JOYBUTTONDOWN) key_value = 1;
		if(event.type == SDL_JOYBUTTONUP) key_value = 0;
		
		// Emulate x86 scancodes
		if(event.type == SDL_KEYDOWN || event.type == SDL_KEYUP)
		{					
			switch(event.key.keysym.sym)
			{
				case SDLK_UP:
					key_scan = SC_UP;	
					break;
				case SDLK_DOWN:
					key_scan = SC_DOWN;
					break;
				case SDLK_LEFT:
					key_scan = SC_LEFT;
					break;				
				case SDLK_RIGHT:
					key_scan = SC_RIGHT;
					break;					
				case SDLK_ESCAPE:
					key_scan = SC_ESCAPE;
					break;
				case SDLK_RETURN: // ENTER
					key_scan = SC_ENTER;
					break;					
				case SDLK_LCTRL:
				case SDLK_LALT:
				case SDLK_LSHIFT:				
				case SDLK_SPACE:
					key_scan = SC_SPACE;
					break;	
				case SDLK_s:
					key_scan = SC_S;
					break;
				case SDLK_f:
					key_scan = SC_F;
					break;						
				default:; // <- a typo?
					break;
			}			
			if(key_scan != -1) Keys[key_scan] = key_value;
		}
		if(event.type == SDL_JOYBUTTONDOWN || event.type == SDL_JOYBUTTONUP)
		{
			switch(event.jbutton.button)
			{
				case BUTTON_UP:
					key_scan = SC_UP;
					break;
				case BUTTON_DOWN:
					key_scan = SC_DOWN;
					break;
				case BUTTON_LEFT:
					key_scan = SC_LEFT;
					break;
				case BUTTON_RIGHT:
					key_scan = SC_RIGHT;
					break;
				case BUTTON_ESCAPE:
					key_scan = SC_ESCAPE;
					break;
				case BUTTON_RETURN:
					key_scan = SC_ENTER;
					break;
				case BUTTON_SHOOT_A:
				case BUTTON_SHOOT_B:
				case BUTTON_SHOOT_C:
				case BUTTON_SHOOT_D:
					key_scan = SC_SPACE;
					break;
				case BUTTON_SCALE:
					key_scan = SC_S;
					break;
				case BUTTON_FULLSCREEN:
					key_scan = SC_F;
					break;
				default:
					break;
			}
			if(key_scan != -1) Keys[key_scan] = key_value;
		}
		if(event.type == SDL_JOYHATMOTION)
		{
			switch(event.jhat.value)
			{
				case SDL_HAT_CENTERED:
					Keys[SC_UP] = 0;
					Keys[SC_DOWN] = 0;
					Keys[SC_LEFT] = 0;
					Keys[SC_RIGHT] = 0;
					break;
				case SDL_HAT_UP:
					Keys[SC_UP] = 1;
					Keys[SC_DOWN] = 0;
					Keys[SC_LEFT] = 0;
					Keys[SC_RIGHT] = 0;
					break;
				case SDL_HAT_DOWN:
					Keys[SC_UP] = 0;
					Keys[SC_DOWN] = 1;
					Keys[SC_LEFT] = 0;
					Keys[SC_RIGHT] = 0;
					break;
				case SDL_HAT_LEFT:
					Keys[SC_UP] = 0;
					Keys[SC_DOWN] = 0;
					Keys[SC_LEFT] = 1;
					Keys[SC_RIGHT] = 0;
					break;
				case SDL_HAT_RIGHT:
					Keys[SC_UP] = 0;
					Keys[SC_DOWN] = 0;
					Keys[SC_LEFT] = 0;
					Keys[SC_RIGHT] = 1;
					break;
				case SDL_HAT_LEFTUP:
					Keys[SC_UP] = 1;
					Keys[SC_DOWN] = 0;
					Keys[SC_LEFT] = 1;
					Keys[SC_RIGHT] = 0;
					break;
				case SDL_HAT_RIGHTUP:
					Keys[SC_UP] = 1;
					Keys[SC_DOWN] = 0;
					Keys[SC_LEFT] = 0;
					Keys[SC_RIGHT] = 1;
					break;
				case SDL_HAT_LEFTDOWN:
					Keys[SC_UP] = 0;
					Keys[SC_DOWN] = 1;
					Keys[SC_LEFT] = 1;
					Keys[SC_RIGHT] = 0;
					break;
				case SDL_HAT_RIGHTDOWN:
					Keys[SC_UP] = 0;
					Keys[SC_DOWN] = 1;
					Keys[SC_LEFT] = 0;
					Keys[SC_RIGHT] = 1;
					break;
			}
		}
		if(event.type == SDL_JOYAXISMOTION)
		{
			switch(event.jaxis.axis)
			{
				/* first analog */
				case 0:
					if(event.jaxis.value < -JOYSTICK_DEADZONE)
					{
						if(!joyReverseAxisX)
						{
							if(joyLastPosX == JOY_POS_RIGHT)
							{
								joyHasMovedX = 0;
							}
						}
						else
						{
							if(joyLastPosX == JOY_POS_LEFT)
							{
								joyHasMovedX = 0;
							}
						}
						if(joyHasMovedX == 0)
						{
							if(!joyReverseAxisX)
							{
								Keys[SC_LEFT] = 1;
								Keys[SC_RIGHT] = 0;
								joyLastPosX = JOY_POS_LEFT;
							}
							else
							{
								Keys[SC_LEFT] = 0;
								Keys[SC_RIGHT] = 1;
								joyLastPosX = JOY_POS_RIGHT;
							}
							joyHasMovedX = 1;
						}
					}
					if(event.jaxis.value > JOYSTICK_DEADZONE)
	                                {
						if(!joyReverseAxisX)
						{
							if(joyLastPosX == JOY_POS_LEFT)
							{
								joyHasMovedX = 0;
							}
						}
						else
						{
							if(joyLastPosX == JOY_POS_RIGHT)
							{
								joyHasMovedX = 0;
							}
						}
						if(joyHasMovedX == 0)
						{
							if(!joyReverseAxisX)
							{
								Keys[SC_LEFT] = 0;
	                                	        	Keys[SC_RIGHT] = 1;
								joyLastPosX = JOY_POS_RIGHT;
							}
							else
							{
								Keys[SC_LEFT] = 1;
								Keys[SC_RIGHT] = 0;
								joyLastPosX = JOY_POS_LEFT;
							}
							joyHasMovedX = 1;
						}
	                                }
					if((event.jaxis.value >= -JOYSTICK_DEADZONE) && (event.jaxis.value < JOYSTICK_DEADZONE))
					{
						Keys[SC_LEFT] = 0;
						Keys[SC_RIGHT] = 0;
						joyHasMovedX = 0;
						joyLastPosX = JOY_POS_CENTER;
					}
					break;
				case 1:
					if(event.jaxis.value < JOYSTICK_DEADZONE)
                                        {
						if(!joyReverseAxisY)
						{
							if(joyLastPosY == JOY_POS_DOWN)
							{
								joyHasMovedY = 0;
							}
						}
						else
						{
							if(joyLastPosY == JOY_POS_UP)
							{
								joyHasMovedY = 0;
							}
						}
						if(joyHasMovedY == 0)
						{
							if(!joyReverseAxisY)
							{
                                                		Keys[SC_UP] = 1;
								Keys[SC_DOWN] = 0;
								joyLastPosY = JOY_POS_UP;
							}
							else
							{
								Keys[SC_UP] = 0;
								Keys[SC_DOWN] = 1;
								joyLastPosY = JOY_POS_DOWN;
							}
							joyHasMovedY = 1;
						}
                                        }
					if(event.jaxis.value > JOYSTICK_DEADZONE)
                                        {
						if(!joyReverseAxisY)
						{
							if(joyLastPosY == JOY_POS_UP)
							{
								joyHasMovedY = 0;
							}
						}
						else
						{
							if(joyLastPosY == JOY_POS_DOWN)
							{
								joyHasMovedY = 0;
							}
						}
						if(joyHasMovedY == 0)
						{
							if(!joyReverseAxisY)
							{
								Keys[SC_UP] = 0;
                                                		Keys[SC_DOWN] = 1;
								joyLastPosY = JOY_POS_DOWN;
							}
							else
							{
								Keys[SC_UP] = 1;
								Keys[SC_DOWN] = 0;
								joyLastPosY = JOY_POS_UP;
							}
							joyHasMovedY = 1;
						}
					}
					if((event.jaxis.value >= -JOYSTICK_DEADZONE) && (event.jaxis.value < JOYSTICK_DEADZONE))
					{
						Keys[SC_UP] = 0;
						Keys[SC_DOWN] = 0;
						joyHasMovedY = 0;
						joyLastPosY = JOY_POS_CENTER;
					}
				/* second analog */
				case 2:
					break;
				case 3:
					break;
			}
		}
		#endif
    } 
	
	// toggle sizes x1 or x2 with scanlines
	//#ifndef __DINGOO__
	#if !defined(__DINGOO__) && !defined(__DINGUX__) && !defined(__GP2X__) && !defined(__CAANOO__)
	if(Keys[SC_F] == 1) {
		fullscr ^= SDL_FULLSCREEN; 
		Keys[SC_F] = 0; 
		LM_GFX_SetScale(2);
		return 0;
	}
	if(Keys[SC_S] == 1 && fullscr == 0) {Keys[SC_S] = 0; _toggle ^= 1; LM_GFX_SetScale(_toggle + 1);}	
	#endif

	return 0;
}

int LM_GFX_Init()
{
	int i;
	int flags = SDL_INIT_VIDEO | SDL_INIT_AUDIO;

	#if defined(__JOYSTICK__)
	flags |= SDL_INIT_JOYSTICK;
	#endif	

	//Start SDL 
	//if(SDL_Init(SDL_INIT_EVERYTHING) < 0) return 0; 
	if(SDL_Init(flags) < 0) return 0; 
	atexit(SDL_Quit); 

	//Set up joystick
	#if defined(__JOYSTICK__)
	if(SDL_NumJoysticks() == 1)
	{
		printf("1 joystick was found:\n");
	}
	else if(SDL_NumJoysticks() > 1)
	{
		printf("%d joysticks were found:\n", SDL_NumJoysticks());
	}
	else
	{
		printf("No joysticks were found\n");
	}
	for(i = 0; i < SDL_NumJoysticks(); i++)
	{
		SDL_JoystickOpen(i);
		printf("  Joy %d) %s\n", i, SDL_JoystickName(i));
	}

	SDL_JoystickEventState(SDL_ENABLE);
	#endif
	
	//Set up screen
	if(bpp == 8)
	{
		screen = SDL_SetVideoMode(320 * scale2x, 240 * scale2x, bpp, SDL_SWSURFACE | fullscr);
	}
	else
	{
		screen = SDL_CreateRGBSurface(SDL_SWSURFACE, 320 * scale2x, 240 * scale2x, 8, 0, 0, 0, 0);
		hardwareScreen = SDL_SetVideoMode(320 * scale2x, 240 * scale2x, bpp, SDL_SWSURFACE | fullscr);
	}
	
	// set up palette (rgba format)
	//SDL_SetPalette(screen, SDL_PHYSPAL, (SDL_Color *)pal256, 0, 256);
	SDL_SetPalette(screen, SDL_LOGPAL, (SDL_Color *)pal256, 0, 256);

	//Hide mouse cursor
	SDL_ShowCursor(0);

	//Set the window caption
	SDL_WM_SetCaption("The Last Mission SDL remake", NULL); 
	
	return 1;
}

void LM_GFX_Deinit()
{
	//SDL_Quit();
}

void LM_GFX_Flip(unsigned char *p)
{

	if(scale2x == 2)
	{
		for(int y = 0; y <= 199; y++)
		{
			for(int x = 0; x <= 319; x++)
			{
				unsigned char tmp = *(unsigned char *)(p + y*320+x);
				unsigned char *pDest = (unsigned char *)screen->pixels;

				*(pDest + 640*40 + (y*2+1)*640+x*2) = tmp;
				*(pDest + 640*40 + (y*2+1)*640+x*2+1) = tmp;
				// dirty scanlines :)
				if((tmp <= 8) && (tmp != 0)) tmp += 8;
				*(pDest + 640*40 + y*2*640+x*2) = tmp;
				*(pDest + 640*40 + y*2*640+x*2+1) = tmp;
			}
		}

	}
	else
	{
		memcpy(screen->pixels + 320*20, p, 320*200);
	}

	//Update Screen 
	if(bpp == 8)
	{
		SDL_Flip(screen);
	}
	else
	{
		SDL_BlitSurface(screen, NULL, hardwareScreen, NULL);
		SDL_Flip(hardwareScreen);
	}
}

void LM_GFX_WaitVSync()
{
}

void LM_GFX_SetScale(int param)
{
	scale2x = ((param - 1) & 1) + 1; // ensure param to be 1 or 2 strictly
	if(fullscr == SDL_FULLSCREEN) scale2x = 2;

	if(bpp == 8)
	{
		screen = SDL_SetVideoMode(320 * scale2x, 240 * scale2x, bpp, SDL_SWSURFACE | fullscr);	
	}
	else
	{
		screen = SDL_CreateRGBSurface(SDL_SWSURFACE | fullscr, 320 * scale2x, 240 * scale2x, 8, 0, 0, 0, 0);
		hardwareScreen = SDL_SetVideoMode(320 * scale2x, 240 * scale2x, bpp, SDL_SWSURFACE | fullscr);
	}
	SDL_SetPalette(screen, SDL_PHYSPAL, (SDL_Color *)pal256, 0, 256);
	LM_ResetKeys();
}

unsigned char pal256[1024] = {
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xAA, 0x00, 0x00, 0xAA, 0x00, 0x00, 0x00, 0xAA, 0xAA, 0x00, 
	 0xAA, 0x00, 0x00, 0x00, 0xAA, 0x00, 0xAA, 0x00, 0xAA, 0x55, 0x00, 0x00, 0xAA, 0xAA, 0xAA, 0x00, 
	 0x55, 0x55, 0x55, 0x00, 0x55, 0x55, 0xFF, 0x00, 0x55, 0xFF, 0x55, 0x00, 0x55, 0xFF, 0xFF, 0x00, 
	 0xFF, 0x55, 0x55, 0x00, 0xFF, 0x55, 0xFF, 0x00, 0xFF, 0xFF, 0x55, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 
	 0x00, 0x00, 0x00, 0x00, 0x14, 0x14, 0x14, 0x00, 0x20, 0x20, 0x20, 0x00, 0x2C, 0x2C, 0x2C, 0x00, 
	 0x38, 0x38, 0x38, 0x00, 0x44, 0x44, 0x44, 0x00, 0x50, 0x50, 0x50, 0x00, 0x61, 0x61, 0x61, 0x00, 
	 0x71, 0x71, 0x71, 0x00, 0x81, 0x81, 0x81, 0x00, 0x91, 0x91, 0x91, 0x00, 0xA1, 0xA1, 0xA1, 0x00, 
	 0xB6, 0xB6, 0xB6, 0x00, 0xCA, 0xCA, 0xCA, 0x00, 0xE2, 0xE2, 0xE2, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 
	 0x00, 0x00, 0xFF, 0x00, 0x40, 0x00, 0xFF, 0x00, 0x7D, 0x00, 0xFF, 0x00, 0xBE, 0x00, 0xFF, 0x00, 
	 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xBE, 0x00, 0xFF, 0x00, 0x7D, 0x00, 0xFF, 0x00, 0x40, 0x00, 
	 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x40, 0x00, 0x00, 0xFF, 0x7D, 0x00, 0x00, 0xFF, 0xBE, 0x00, 0x00, 
	 0xFF, 0xFF, 0x00, 0x00, 0xBE, 0xFF, 0x00, 0x00, 0x7D, 0xFF, 0x00, 0x00, 0x40, 0xFF, 0x00, 0x00, 
	 0x00, 0xFF, 0x00, 0x00, 0x00, 0xFF, 0x40, 0x00, 0x00, 0xFF, 0x7D, 0x00, 0x00, 0xFF, 0xBE, 0x00, 
	 0x00, 0xFF, 0xFF, 0x00, 0x00, 0xBE, 0xFF, 0x00, 0x00, 0x7D, 0xFF, 0x00, 0x00, 0x40, 0xFF, 0x00, 
	 0x7D, 0x7D, 0xFF, 0x00, 0x9D, 0x7D, 0xFF, 0x00, 0xBE, 0x7D, 0xFF, 0x00, 0xDE, 0x7D, 0xFF, 0x00, 
	 0xFF, 0x7D, 0xFF, 0x00, 0xFF, 0x7D, 0xDE, 0x00, 0xFF, 0x7D, 0xBE, 0x00, 0xFF, 0x7D, 0x9D, 0x00, 
	 0xFF, 0x7D, 0x7D, 0x00, 0xFF, 0x9D, 0x7D, 0x00, 0xFF, 0xBE, 0x7D, 0x00, 0xFF, 0xDE, 0x7D, 0x00, 
	 0xFF, 0xFF, 0x7D, 0x00, 0xDE, 0xFF, 0x7D, 0x00, 0xBE, 0xFF, 0x7D, 0x00, 0x9D, 0xFF, 0x7D, 0x00, 
	 0x7D, 0xFF, 0x7D, 0x00, 0x7D, 0xFF, 0x9D, 0x00, 0x7D, 0xFF, 0xBE, 0x00, 0x7D, 0xFF, 0xDE, 0x00, 
	 0x7D, 0xFF, 0xFF, 0x00, 0x7D, 0xDE, 0xFF, 0x00, 0x7D, 0xBE, 0xFF, 0x00, 0x7D, 0x9D, 0xFF, 0x00, 
	 0xB6, 0xB6, 0xFF, 0x00, 0xC6, 0xB6, 0xFF, 0x00, 0xDA, 0xB6, 0xFF, 0x00, 0xEA, 0xB6, 0xFF, 0x00, 
	 0xFF, 0xB6, 0xFF, 0x00, 0xFF, 0xB6, 0xEA, 0x00, 0xFF, 0xB6, 0xDA, 0x00, 0xFF, 0xB6, 0xC6, 0x00, 
	 0xFF, 0xB6, 0xB6, 0x00, 0xFF, 0xC6, 0xB6, 0x00, 0xFF, 0xDA, 0xB6, 0x00, 0xFF, 0xEA, 0xB6, 0x00, 
	 0xFF, 0xFF, 0xB6, 0x00, 0xEA, 0xFF, 0xB6, 0x00, 0xDA, 0xFF, 0xB6, 0x00, 0xC6, 0xFF, 0xB6, 0x00, 
	 0xB6, 0xFF, 0xB6, 0x00, 0xB6, 0xFF, 0xC6, 0x00, 0xB6, 0xFF, 0xDA, 0x00, 0xB6, 0xFF, 0xEA, 0x00, 
	 0xB6, 0xFF, 0xFF, 0x00, 0xB6, 0xEA, 0xFF, 0x00, 0xB6, 0xDA, 0xFF, 0x00, 0xB6, 0xC6, 0xFF, 0x00, 
	 0x00, 0x00, 0x71, 0x00, 0x1C, 0x00, 0x71, 0x00, 0x38, 0x00, 0x71, 0x00, 0x55, 0x00, 0x71, 0x00, 
	 0x71, 0x00, 0x71, 0x00, 0x71, 0x00, 0x55, 0x00, 0x71, 0x00, 0x38, 0x00, 0x71, 0x00, 0x1C, 0x00, 
	 0x71, 0x00, 0x00, 0x00, 0x71, 0x1C, 0x00, 0x00, 0x71, 0x38, 0x00, 0x00, 0x71, 0x55, 0x00, 0x00, 
	 0x71, 0x71, 0x00, 0x00, 0x55, 0x71, 0x00, 0x00, 0x38, 0x71, 0x00, 0x00, 0x1C, 0x71, 0x00, 0x00, 
	 0x00, 0x71, 0x00, 0x00, 0x00, 0x71, 0x1C, 0x00, 0x00, 0x71, 0x38, 0x00, 0x00, 0x71, 0x55, 0x00, 
	 0x00, 0x71, 0x71, 0x00, 0x00, 0x55, 0x71, 0x00, 0x00, 0x38, 0x71, 0x00, 0x00, 0x1C, 0x71, 0x00, 
	 0x38, 0x38, 0x71, 0x00, 0x44, 0x38, 0x71, 0x00, 0x55, 0x38, 0x71, 0x00, 0x61, 0x38, 0x71, 0x00, 
	 0x71, 0x38, 0x71, 0x00, 0x71, 0x38, 0x61, 0x00, 0x71, 0x38, 0x55, 0x00, 0x71, 0x38, 0x44, 0x00, 
	 0x71, 0x38, 0x38, 0x00, 0x71, 0x44, 0x38, 0x00, 0x71, 0x55, 0x38, 0x00, 0x71, 0x61, 0x38, 0x00, 
	 0x71, 0x71, 0x38, 0x00, 0x61, 0x71, 0x38, 0x00, 0x55, 0x71, 0x38, 0x00, 0x44, 0x71, 0x38, 0x00, 
	 0x38, 0x71, 0x38, 0x00, 0x38, 0x71, 0x44, 0x00, 0x38, 0x71, 0x55, 0x00, 0x38, 0x71, 0x61, 0x00, 
	 0x38, 0x71, 0x71, 0x00, 0x38, 0x61, 0x71, 0x00, 0x38, 0x55, 0x71, 0x00, 0x38, 0x44, 0x71, 0x00, 
	 0x50, 0x50, 0x71, 0x00, 0x59, 0x50, 0x71, 0x00, 0x61, 0x50, 0x71, 0x00, 0x69, 0x50, 0x71, 0x00, 
	 0x71, 0x50, 0x71, 0x00, 0x71, 0x50, 0x69, 0x00, 0x71, 0x50, 0x61, 0x00, 0x71, 0x50, 0x59, 0x00, 
	 0x71, 0x50, 0x50, 0x00, 0x71, 0x59, 0x50, 0x00, 0x71, 0x61, 0x50, 0x00, 0x71, 0x69, 0x50, 0x00,  
	 0x71, 0x71, 0x50, 0x00, 0x69, 0x71, 0x50, 0x00, 0x61, 0x71, 0x50, 0x00, 0x59, 0x71, 0x50, 0x00, 
	 0x50, 0x71, 0x50, 0x00, 0x50, 0x71, 0x59, 0x00, 0x50, 0x71, 0x61, 0x00, 0x50, 0x71, 0x69, 0x00, 
	 0x50, 0x71, 0x71, 0x00, 0x50, 0x69, 0x71, 0x00, 0x50, 0x61, 0x71, 0x00, 0x50, 0x59, 0x71, 0x00, 
	 0x00, 0x00, 0x40, 0x00, 0x10, 0x00, 0x40, 0x00, 0x20, 0x00, 0x40, 0x00, 0x30, 0x00, 0x40, 0x00, 
	 0x40, 0x00, 0x40, 0x00, 0x40, 0x00, 0x30, 0x00, 0x40, 0x00, 0x20, 0x00, 0x40, 0x00, 0x10, 0x00, 
	 0x40, 0x00, 0x00, 0x00, 0x40, 0x10, 0x00, 0x00, 0x40, 0x20, 0x00, 0x00, 0x40, 0x30, 0x00, 0x00, 
	 0x40, 0x40, 0x00, 0x00, 0x30, 0x40, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 0x10, 0x40, 0x00, 0x00, 
	 0x00, 0x40, 0x00, 0x00, 0x00, 0x40, 0x10, 0x00, 0x00, 0x40, 0x20, 0x00, 0x00, 0x40, 0x30, 0x00, 
	 0x00, 0x40, 0x40, 0x00, 0x00, 0x30, 0x40, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 0x10, 0x40, 0x00, 
	 0x20, 0x20, 0x40, 0x00, 0x28, 0x20, 0x40, 0x00, 0x30, 0x20, 0x40, 0x00, 0x38, 0x20, 0x40, 0x00, 
	 0x40, 0x20, 0x40, 0x00, 0x40, 0x20, 0x38, 0x00, 0x40, 0x20, 0x30, 0x00, 0x40, 0x20, 0x28, 0x00, 
	 0x40, 0x20, 0x20, 0x00, 0x40, 0x28, 0x20, 0x00, 0x40, 0x30, 0x20, 0x00, 0x40, 0x38, 0x20, 0x00, 
	 0x40, 0x40, 0x20, 0x00, 0x38, 0x40, 0x20, 0x00, 0x30, 0x40, 0x20, 0x00, 0x28, 0x40, 0x20, 0x00, 
	 0x20, 0x40, 0x20, 0x00, 0x20, 0x40, 0x28, 0x00, 0x20, 0x40, 0x30, 0x00, 0x20, 0x40, 0x38, 0x00, 
	 0x20, 0x40, 0x40, 0x00, 0x20, 0x38, 0x40, 0x00, 0x20, 0x30, 0x40, 0x00, 0x20, 0x28, 0x40, 0x00, 
	 0x2C, 0x2C, 0x40, 0x00, 0x30, 0x2C, 0x40, 0x00, 0x34, 0x2C, 0x40, 0x00, 0x3C, 0x2C, 0x40, 0x00, 
	 0x40, 0x2C, 0x40, 0x00, 0x40, 0x2C, 0x3C, 0x00, 0x40, 0x2C, 0x34, 0x00, 0x40, 0x2C, 0x30, 0x00, 
	 0x40, 0x2C, 0x2C, 0x00, 0x40, 0x30, 0x2C, 0x00, 0x40, 0x34, 0x2C, 0x00, 0x40, 0x3C, 0x2C, 0x00, 
	 0x40, 0x40, 0x2C, 0x00, 0x3C, 0x40, 0x2C, 0x00, 0x34, 0x40, 0x2C, 0x00, 0x30, 0x40, 0x2C, 0x00, 
	 0x2C, 0x40, 0x2C, 0x00, 0x2C, 0x40, 0x30, 0x00, 0x2C, 0x40, 0x34, 0x00, 0x2C, 0x40, 0x3C, 0x00, 
	 0x2C, 0x40, 0x40, 0x00, 0x2C, 0x3C, 0x40, 0x00, 0x2C, 0x34, 0x40, 0x00, 0x2C, 0x30, 0x40, 0x00, 
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 
};
